#property copyright "Copyright  2016, Marketeer"
#property link      "https://www.mql5.com/en/users/marketeer"
#property version   "1.0"
#property description "Trade Player Example: executes trade operations according to provided CSV log\n"
#property description "It allows you to convert a sequence of signals stored during multicurrency EA test into orders of any symbols other than work symbol used initially during EA test, which saved the CSV log."

#property strict

extern string TradeRecordsFile = "";
extern int    Deviation = 10;
extern bool   MultipleOrders = false;
extern bool   BarByBar = true;


#define DIRECTION_IN 1
#define DIRECTION_OUT -1

struct DumpRecord
{
  int type;
  int direction;
  double lot;
  string symbol;
  datetime time;
};

class Filer
{
  protected:
    int dump;
    
  public:
    Filer()
    {
      dump = INVALID_HANDLE;
    }
    
    virtual ~Filer()
    {
      if(dump != INVALID_HANDLE)
      {
        FileClose(dump);
      }
    }
    
};

/*

EXAMPLE of code for writing trade operations to CSV

class Dumper: public Filer
{
  public:
    Dumper(const string name)
    {
      dump = FileOpen(name, FILE_WRITE | FILE_CSV | FILE_SHARE_WRITE);
    }

    void dumpTrade(const int type, const int direction, const double lot, const string symbol, const datetime now = 0) const
    {
      if(dump != INVALID_HANDLE)
      {
        FileWrite(dump, TimeToStr(now == 0?TimeCurrent():now), type, direction, DoubleToString(lot, 2), symbol);
      }
    }
};

Dumper writer("file.csv");
...
writer.dumpTrade(OP_BUY, DIRECTION_IN, 0.1, AnotherSymbol());

*/

// EXAMPLE of code for reading operations from CSV and executing them as market orders

class DumpReader: public Filer
{
  private:
    DumpRecord record;
    
  protected:
    void readCSVLine()
    {
      // read in the loop until find a record for Symbol() next to current time (in near future)
      while(!FileIsEnding(dump) && (record.time < TimeCurrent() || record.symbol != Symbol()))
      {
        record.time = FileReadDatetime(dump);
        record.type = (int)FileReadNumber(dump);
        record.direction = (int)FileReadNumber(dump);
        record.lot = FileReadNumber(dump);
        record.symbol = FileReadString(dump);
        Print("New record read:", (string)record.time, " ", (string)record.type, " ", (string)record.direction, " ", FileIsEnding(dump));
      }
    }
  
  public:
    DumpReader()
    {
      record.time = 0;
    }
    
    DumpReader(const string name)
    {
      dump = FileOpen(name, FILE_READ | FILE_CSV | FILE_SHARE_READ);
      record.time = 0;
    }
    
    bool advance()
    {
      if(record.time == 0)
      {
        readCSVLine();
      }
      return record.time != 0; // has pending signal
    }
    
    datetime getTime() const
    {
      return record.time;
    }

    int getType() const
    {
      return record.type;
    }

    int getDirection() const
    {
      return record.direction;
    }

    double getLot() const
    {
      return record.lot;
    }
    
    bool isReady() const
    {
      return dump != INVALID_HANDLE;
    }
    
    void reset()
    {
      record.type = -1;
      record.time = 0;
    }

    static bool OrderSelectLast(const int op)
    {
      for(int i = OrdersTotal() - 1; i >= 0; i--)
      {
        if(OrderSelect(i, SELECT_BY_POS))
        {
          if(OrderType() == op)
          {
            return(true);
          }
        }
      }
      return(false);
    }
    
};

DumpReader reader(TradeRecordsFile);
  


int init()
{
  if(!reader.isReady()) return(INIT_PARAMETERS_INCORRECT);

  return(INIT_SUCCEEDED);
}

int deinit()
{
  return(0);
}

int start()
{
  static datetime lastBarTime;
  if((lastBarTime == Time[0]) && BarByBar) return 0;
  lastBarTime = Time[0];
  
  while(reader.advance() && TimeCurrent() >= reader.getTime())
  {
    // as soon as time passes by pending signal from the record, execute it  
    {
      int signal = reader.getType();
      
      // TODO: support pending orders (use corresponding OP_XXX to pass specific price to OrderSend)
      // TODO: support SL/TP
      // TODO: support pending orders deletion via OrderDelete

      if(signal != -1 && signal < 2) // OP_BUY/OP_SELL
      {
        if(reader.getDirection() == DIRECTION_IN)
        {
          if(MultipleOrders || !reader.OrderSelectLast(signal))
          {
            if(OrderSend(Symbol(), signal, reader.getLot(), signal == OP_BUY?Ask:Bid, Deviation, 0, 0) == -1)
            {
              Print("OrderSend failed:", GetLastError());
            }
          }
        }
        else
        if(reader.getDirection() == DIRECTION_OUT)
        {
          if(reader.OrderSelectLast(signal))
          {
            if(reader.getLot() > 0 && OrderLots() != reader.getLot())
            {
              // TODO: find more open orders or use OrderCloseBy
              Print("Stub: not implemeted");
            }
            else
            {
              if(!OrderClose(OrderTicket(), OrderLots(), signal == OP_BUY?Bid:Ask, Deviation))
              {
                Print("OrderClose failed:", GetLastError());
              }
            }
          }
        }
      }
      reader.reset();
    }
  }
      
  return 0;
}
